﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using PaintDotNet.Effects;
using PaintDotNet;

namespace Controlz
{
    public partial class ColorWheelTR : UserControl
    {
        int _hue = 240;
        int _sat = 100;
        int _val = 100;
        int _alph =255;
        Color _colorval = Color.Blue;
        bool colorpicked ;
        bool ignore = false;
        Bitmap bmp = new Bitmap(200, 100);

        public int hue
        {
            get { return _hue; }
        }
        public int sat
        {
            get { return _sat; }
        }
        
        public int val
        {
            get { return _val; }
        }
        public int alph
        {
            get { return _alph; }
        }

        public Color colorval
        {
            get { return _colorval; }
            set
            {
                _colorval = value;
                setColors();
                OnValueChanged(_colorval);
            }
        }

        public delegate void ValueChangedEventHandler(object sender, Color e);
        public event ValueChangedEventHandler ValueChanged;

        protected  void OnValueChanged(Color e)
        {
            if (this.ValueChanged != null) this.ValueChanged(this, e);
        } 



        public ColorWheelTR()
        {
            InitializeComponent();
        }


        private void GenColorWheel_Paint()
        {
            Rectangle myrect = new Rectangle(0,0,200,100);
            int slim = myrect.Width / 10;

            Rectangle wrect = new Rectangle(0, 0, myrect.Width / 2, myrect.Height);
            Rectangle mrect = new Rectangle(-1, -1, myrect.Width / 2 + 2, myrect.Height + 2);
            Rectangle Lrect = new Rectangle((int)(5.5f * slim), 0, slim, myrect.Height);
            Rectangle Arect = new Rectangle((int)(7.5f * slim), 0, slim, myrect.Height);
            Rectangle Crect = new Rectangle((int)(9 * slim) - 1, 0, slim, slim);


            GraphicsPath wheel_path = new GraphicsPath();
            wheel_path.AddEllipse(wrect);
            wheel_path.Flatten();

            float num_pts = wheel_path.PointCount;
            Color[] surround_colors = new Color[wheel_path.PointCount];
            for (float i = 0; i < num_pts; i++)
            {
                HsvColor hsv = new HsvColor((int)(i * 360 / num_pts), 100, 100);
                Color c = hsv.ToColor();
                surround_colors[(int)i] = c;
            }

            using (Graphics g = Graphics.FromImage(bmp))
            {
               
                g.SmoothingMode = SmoothingMode.AntiAlias;
                g.FillRectangle(new SolidBrush(this.BackColor), 0, 0, 200, 100);
                using (Pen pen = new Pen(this.BackColor, 2f))
                {
                    g.DrawEllipse(pen, mrect);
                }
                using (PathGradientBrush path_brush = new PathGradientBrush(wheel_path))
                {
                    path_brush.CenterColor = Color.White;
                    path_brush.SurroundColors = surround_colors;

                    g.FillPath(path_brush, wheel_path);
                    using (Pen thick_pen = new Pen(this.BackColor, 1.5f))
                    {
                        g.DrawPath(thick_pen, wheel_path);
                    }
                }



                //Draw Lightness Control
                HsvColor hsv2 = new HsvColor(_hue, _sat, 100);
                using (LinearGradientBrush lgb = new LinearGradientBrush(Lrect, hsv2.ToColor(), Color.Black, LinearGradientMode.Vertical))
                {
                    Pen pen = new Pen(lgb);
                    g.FillRectangle(lgb, Lrect);
                    pen.Dispose();
                }

                //draw alpha control
                HatchBrush hb = new HatchBrush(HatchStyle.LargeCheckerBoard, Color.LightGray, Color.White);
                g.FillRectangle(hb, Arect);

                using (LinearGradientBrush lgb = new LinearGradientBrush(Arect, hsv2.ToColor(), Color.Transparent, LinearGradientMode.Vertical))
                {
                    Pen pen = new Pen(lgb);
                    g.FillRectangle(lgb, Arect);
                    pen.Dispose();
                }

                //draw colorsample
                hsv2.Value = _val;
                Color CB = Color.FromArgb(_alph, hsv2.ToColor());

                SolidBrush SB = new SolidBrush(CB);
                g.FillRectangle(hb, Crect);
                g.FillRectangle(SB, Crect);
                g.DrawRectangle(Pens.Black, Crect);
                hb.Dispose();

                // set val markers
                int valtop = myrect.Height - _val * myrect.Height / 100;
                Rectangle valMark = new Rectangle(Lrect.Left, valtop, Lrect.Width, 3);
                using (Pen valPen = new Pen(Color.FromArgb(255, 255 - _val, 255 - _val, 255 - _val)))
                {
                    g.DrawRectangle(valPen, valMark);
                }


                // set alpha markers
                int alphtop = myrect.Height - _alph * myrect.Height / 255;
                Rectangle alphMark = new Rectangle(Arect.Left, alphtop, slim, 3);
                g.DrawRectangle(Pens.Black, alphMark);

                //set _hue and sat marker
                int hlfht = myrect.Height / 2;
                double radius = _sat * (hlfht * .98) / 100;
                int _hueX = (int)(hlfht + radius * Math.Cos(_hue * Math.PI / 180));
                int _huey = (int)(hlfht + radius * Math.Sin(_hue * Math.PI / 180));
                Rectangle _hueMark = new Rectangle(_hueX - 2, _huey - 2, 5, 5);
                g.FillEllipse(new SolidBrush(Color.Black), _hueMark);
            }

        }

        private void GenColorWheel_MouseDown(object sender, MouseEventArgs e)
        {
            Panel p = (Panel)sender;
            colorpicked = true;
            GenColorWheel_MouseMove(sender, e);
        }

        private void GenColorWheel_MouseMove(object sender, MouseEventArgs e)
        {
            Panel p = (Panel)sender;
            if (!colorpicked) return;

            Rectangle myrect = p.ClientRectangle;
            int slim = myrect.Width / 10;
            int hlfht = myrect.Height / 2;
            double offset;

            Rectangle wrect = new Rectangle(0, 0, myrect.Width / 2, myrect.Height);
            Rectangle Lrect = new Rectangle((int)(5.5f * slim), 0, slim, myrect.Height);
            Rectangle Arect = new Rectangle((int)(7.5f * slim), 0, slim, myrect.Height);

            if (wrect.Contains(e.Location))
            {
                double rad = (Math.Atan2(e.Y - hlfht, e.X - hlfht)) * 180 / Math.PI;
                rad = (rad < 0) ? rad + 360 : rad;
                _hue = (int)rad;
                offset = Math.Sqrt((e.Y - hlfht) * (e.Y - hlfht) + (e.X - hlfht) * (e.X - hlfht)) / hlfht * 100;
                offset = (offset > 100) ? 100 : offset;
                _sat = (int)offset;

            }
            else if (Lrect.Contains(e.Location))
            {
                offset = 100 - (e.Y - Lrect.Y) * 100 / Lrect.Height;
                _val = (int)offset;
            }
            else if (Arect.Contains(e.Location))
            {
                offset = 255 - (e.Y - Arect.Y) * 255 / Arect.Height;
                _alph = (int)offset;
            }
            HsvColor hsv2 = new HsvColor(_hue, _sat, _val);
            _colorval = Color.FromArgb(_alph, hsv2.ToColor());
            setColors();
            OnValueChanged(_colorval);
            this.Refresh();
        }

        private void GenColorWheel_MouseUp(object sender, MouseEventArgs e)
        {
            Panel p = (Panel)sender;
            OnValueChanged(_colorval);
            colorpicked = false;
            OnValueChanged(_colorval);
            this.Refresh();
        }
        private void setColors()
        {
            ignore = true;
            HsvColor hsv = HsvColor.FromColor(_colorval);
            _hue = hsv.Hue;
            _val = hsv.Value;
            _sat = hsv.Saturation;
            _alph = _colorval.A;
            red1.Value = _colorval.R;
            green1.Value = _colorval.G;
            blue1.Value = _colorval.B;
            alpha1.Value = _colorval.A;
            ignore = false;

        }
        private void rgb_ValueChanged(object sender, EventArgs e)
        {
            if (ignore) return;
            NumericUpDown p = (NumericUpDown)sender;
            _colorval = Color.FromArgb((int)alpha1.Value, (int)red1.Value, (int)green1.Value, (int)blue1.Value);
            HsvColor hsv = HsvColor.FromColor(_colorval);
            _hue = hsv.Hue;
            _val = hsv.Value;
            _sat = hsv.Saturation;
            _alph = _colorval.A;
            this.Refresh();
            OnValueChanged(_colorval);
        }
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            e.Graphics.FillRectangle(new SolidBrush(this.BackColor), 200, 0, 98, 100);
        }
        protected override void OnPaint(PaintEventArgs e)
        {
            GenColorWheel_Paint();
            e.Graphics.DrawImage(bmp, 0, 0);
        }

        private void ColorWheelTR_Load(object sender, EventArgs e)
        {

        }

        private void panel2_Paint(object sender, PaintEventArgs e)
        {

        }
    }
}
